#include <assert.h>
#include <string.h>

#include "array.h"
#include "process.h"

void test_simple_case() {
    process *processes = NULL;
    parse_line("cat my_txt   ", &processes);
    assert(arrayLen(processes) == 1);
    assert(arrayLen(processes[0].argv) == 3);
    assert(strcmp(processes[0].argv[0], "cat") == 0);
    assert(strcmp(processes[0].argv[1], "my_txt") == 0);
    free_processes(&processes);
}

void test_detached() {
    process *processes = NULL;
    parse_line("cat my_txt &", &processes);
    assert(arrayLen(processes) == 1);
    assert(arrayLen(processes[0].argv) == 3);
    assert(strcmp(processes[0].argv[0], "cat") == 0);
    assert(strcmp(processes[0].argv[1], "my_txt") == 0);
    assert(processes[0].blocking == false);
    free_processes(&processes);
}


void test_pipe() {
    process *processes = NULL;
    parse_line("echo my_txt | grep txt", &processes);
    assert(arrayLen(processes) == 2);
    assert(arrayLen(processes[0].argv) == 3);
    assert(strcmp(processes[0].argv[0], "echo") == 0);
    assert(strcmp(processes[0].argv[1], "my_txt") == 0);
    assert(processes[0].out_fd != 0);
    assert(arrayLen(processes[1].argv) == 3);
    assert(strcmp(processes[1].argv[0], "grep") == 0);
    assert(strcmp(processes[1].argv[1], "txt") == 0);
    assert(processes[1].in_fd != 0);
    free_processes(&processes);
}

void test_ls(){
    process *processes = NULL;
    parse_line("ls", &processes);
    assert(arrayLen(processes) == 1);
    process p = processes[0];
    assert(arrayLen(p.argv)==2);
    assert(p.argc == 1);
    assert(strcmp(p.argv[0], "ls") == 0);
    assert(p.argc[p.argv] == NULL);
    free_processes(&processes);
}

void test_multiparse() {
    Process *processes = NULL;
    parse_line("echo abcdefg", &processes);
    assert(arrayLen(processes) == 1);
    Process p = processes[0];
    assert(arrayLen(p.argv)==3);
    assert(p.argc == 2);
    assert(strcmp(p.argv[0], "echo") == 0);
    assert(strcmp(p.argv[1], "abcdefg") == 0);
    assert(p.argc[p.argv] == NULL);
    free_processes(&processes);
    parse_line("echo abc", &processes);
    assert(arrayLen(processes) == 1);
    p = processes[0];
    assert(arrayLen(p.argv)==3);
    assert(p.argc == 2);
    assert(strcmp(p.argv[0], "echo") == 0);
    assert(strcmp(p.argv[1], "abc") == 0);
    assert(p.argc[p.argv] == NULL);
}


int main() {
    test_ls();
    test_simple_case();
    test_detached();
    test_pipe();
    test_multiparse();
    return 0;
}
